#include <math.h>
#include <stdlib.h>
#include "glut.h"

#include "linmath.h"
#include "texture.h"
#include <stdlib.h>

struct Texture *floorTexture=NULL;
static int animate=0;

static void setupTexture(struct Texture *tx)
{
   glGenTextures(1,&tx->mTextureNum);
   glBindTexture(GL_TEXTURE_2D,tx->mTextureNum);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, 
		   GL_LINEAR);
   glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, 
		   GL_LINEAR);
   glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, tx->mWidth,
		tx->mHeight, 0, GL_RGB, GL_UNSIGNED_BYTE,
		tx->mRGB);
}

static void drawFloor(void)
{
   /* enable texturing and set floorTexture to active */
   glEnable(GL_TEXTURE_2D);	 
   glBindTexture(GL_TEXTURE_2D, floorTexture->mTextureNum);
   glBegin(GL_QUADS);
     glColor3f(0.6, 0.6, 0.6);  /* gray */
     glTexCoord2f(0,0);
     glVertex3f(-4.0,-1.74,-4.0);
     glTexCoord2f(1,0);
     glVertex3f( 4.0,-1.74,-4.0);
     glTexCoord2f(1,1);
     glVertex3f( 4.0,-1.74, 4.0);
     glTexCoord2f(0,1);
     glVertex3f(-4.0,-1.74, 4.0);
   glEnd();

   glDisable(GL_TEXTURE_2D);	 
}

static void drawQuad(void)
{
   static float a=0.0;
   static float b=0.0;
   float col=(cos(a)+1.0)/2.0;
   float col2=(cos(b)+1.0)/2.0;
   glDisable(GL_LIGHTING);  /* no lighting */
   /* draw a quadrilateral with a color at each vertex */
   glBegin(GL_QUADS);
     glColor3f(0.0,1-col2, col2);  
     glVertex3f(-1.0,-1.0,1.0);
     glColor3f(col, 0.0, col);
     glVertex3f( 1.0,-1.0,1.0);
     glColor3f(col2, col, 1-col2);  
     glVertex3f( 1.0, 1.0,1.0);
     glColor3f(1.0, 1.0, 1.0);  /* white */
     glVertex3f(-1.0, 1.0,1.0);
   glEnd();
   a+=0.03;
   b+=0.05;
}

static void drawLines(void)
{
   int q;
   float x,z;
   glBegin(GL_LINES);
     for(q=0;q<10;q++)
     {
	x=2.0*((float)q/9.0-0.5);
	glColor3f(0.8,0.6,0.2);
	glVertex3f(x,  1.0,-1.0);
	glVertex3f(1.0,1.0,-x);

	glColor3f(0.2,0.6,0.8);
	glVertex3f(-x,  1.0,1.0);
	glVertex3f(-1.0,1.0,x);
     }     
   glEnd();
}

static void drawPoints(void)
{
   int q;
   static int init=1;
   static float x[100],y[100];
   const int numpoints=100;
   if(init)
   {
      init=0;
      for(q=0;q<numpoints;q++)
      {
	 x[q]=2.0*(rand()-0.5);
	 y[q]=2.0*(rand()-0.5);
      }
   }

   glBegin(GL_POINTS);
   for(q=0;q<numpoints;q++)
   {
     glColor3f(1,1,1);
     glVertex3f(x[q],y[q],-1.0);
   }
   glEnd();
}

static void setupLight(void)
{
   GLfloat white[3]={1,1,1};
   GLfloat pos[4]={0,2,5,1};
  /* set colors to white for light 0, and set its position */
   glLightfv(GL_LIGHT0, GL_AMBIENT, white);
   glLightfv(GL_LIGHT0, GL_DIFFUSE, white);
   glLightfv(GL_LIGHT0, GL_SPECULAR, white);
   glLightfv(GL_LIGHT0, GL_POSITION, pos);
   glEnable(GL_LIGHTING);  /* enable lighting */
   glEnable(GL_LIGHT0); /* enable light 0 */
}

static void drawLitStuff(void)
{
   GLfloat dark[3]={.2,0.2,0.2};
   GLfloat blue[3]={0.2,0.2,1};
   GLfloat white[3]={1,1,1};
   glEnable(GL_LIGHTING);  /* enable lighting */
   glEnable(GL_LIGHT0); /* enable light 0 */

   /* set material for front and back */
   glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, dark);
   glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, blue);
   glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, white);
   glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, 20);

   glBegin(GL_TRIANGLES);

   glNormal3f(-1,0,0);  /* we need normals because we use lighting */
   glVertex3f(-1,1,-1); /* tri 1 */
   glVertex3f(-1,-1,1);
   glVertex3f(-1,1, 1);

   glVertex3f(-1,1,-1); /* tri 2 (share normal with tri 1) */
   glVertex3f(-1,-1,1);
   glVertex3f(-1,-1,-1);

   glNormal3f(0,-1,0);  
   glVertex3f(1,-1,-1);
   glVertex3f(-1,-1,1);
   glVertex3f(1,-1, 1);

   glVertex3f(1,-1,-1);
   glVertex3f(-1,-1,1);
   glVertex3f(-1,-1,-1);


   glNormal3f(1,0,0);  /* we need normals because we use lighting */
   glVertex3f(1,1,-1); /* tri 1 */
   glVertex3f(1,-1,1);
   glVertex3f(1,1, 1);

   glVertex3f(1,1,-1); /* tri 2 (share normal with tri 1) */
   glVertex3f(1,-1,1);
   glVertex3f(1,-1,-1);

   glEnd();
   
   glDisable(GL_LIGHT0);
   glDisable(GL_LIGHTING);
}

static void drawMyCoolStuff(void)
{
   static double degx=0.0,degy=0.0;
   glPushMatrix();
     glRotated(degx,1,0,0);   /* rotate around the x-axis */
     glRotated(degy,0,1,0);   /* rotate around the y-axis */
     drawQuad();
     drawLines();
     drawPoints();
     drawLitStuff();
   glPopMatrix();
   if(animate)
   {
      degx+=1.12345;
      degy+=2.0;
   }
}

void display(void)
{
    glPopMatrix();
   /* clear screen (Z-buffer and color buffer) */
   glClearColor(0.2,0.2,0.8,1.0);
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
   
   drawFloor(); 
   setupLight(); 
   drawMyCoolStuff();
  
   glutSwapBuffers();    /* swap front and back buffer */
   glutPostRedisplay();
}

void handleKeys(unsigned char key, int x, int y)
{
   switch(key)
   {
   case 27:    /* ESC */
      exit(0); /* dirty exit */
      break;   /* unnecessary, I know */
   case 32:    /* space */
      animate=!animate;
      break;
  }
}

void handleSpecialKeys(int key, int x, int y)
{
   switch(key)
   {
   case GLUT_KEY_LEFT:
      printf("Left arrow\n");
      break;
   case GLUT_KEY_RIGHT:
      printf("Right arrow\n");
      break;
   }
}

/* global variables for navigation */
static int lbut_state = 1;
static int mbut_state = 1;
static int rbut_state = 1;

static float viewpos[] = {0,0,8};
static float viewat[] = {0,0,0};
static float viewup[] = {0,1,0};
static float viewvec[] = {0,0,-8};
static float rotcenter[] = {0,0};

static int prev_x = 0;
static int prev_y = 0;

void mouse(int button, int state, int x, int y)
{
   /* button=GLUT_LEFT_BUTTON, GLUT_MIDDLE_BUTTON, or GLUT_RIGHT_BUTTON, */
   printf("button: %d, state=%d, (x,y)=(%d,%d)\n",button,state,x,y); 

   prev_x = x;
   prev_y = y;

   switch(button)
   {
   case 0:
	   lbut_state = state; /* up/down/left/right */
	   break;
   case 1:
	   mbut_state = state; /* zoom in/out */
	   break;
   case 2: 
	   rbut_state = state; /* rotate around the rotation center */
   default:
	   break;
   }
}


void motion(int x, int y)
{
   int delta_x = x - prev_x;
   int delta_y = y - prev_y;
   const float speed = 0.1;
   float len;
   Vec3f tmpvec, axis;
   Mat33f rotmat;
  
   printf("Motion: %d %d\n",x,y);

   /* your task: compute viewpos, viewat, viewup from mouse movements */
   /* and then load the view matrix with gluLookAt() */
   /* something like: */
   /*
    * glMatrixMode(GL_MODELVIEW);  
    * glLoadIdentity();          	 
    * compute new view params here....
    * gluLookAt(viewpos[X],viewpos[Y],viewpos[Z],
    *           viewat[X],viewat[Y],viewat[Z],
    *           viewup[X],viewup[Y],viewup[Z]);
    */   

   glMatrixMode(GL_MODELVIEW);  
   glLoadIdentity(); 

   if(lbut_state == 0)
   {
 	   /* calculate screen-x axis */
	   V3CROSS(axis, viewup, viewvec);
	   V3NORMALIZEF(axis);
	   V3SCALE(tmpvec, axis, delta_x * speed * 0.3f);
	   V3ADD(viewpos, viewpos, tmpvec);
 	   /* calculate screen-y axis */
	   V3CROSS(tmpvec, viewvec, axis);
	   V3NORMALIZEF(tmpvec);
	   V3SCALE(tmpvec, tmpvec, delta_y * speed * 0.3f);
	   V3ADD(viewpos, viewpos, tmpvec);
 }
   if(mbut_state == 0)
   {
	   V3SET(tmpvec, viewvec);
	   V3NORMALIZEF(tmpvec);
	   viewpos[0] -= delta_y * speed*tmpvec[0]; 
	   viewpos[1] -= delta_y * speed*tmpvec[1]; 
	   viewpos[2] -= delta_y * speed*tmpvec[2]; 
   }
   viewat[X] = viewpos[X]+viewvec[X];
   viewat[Y] = viewpos[Y]+viewvec[Y];
   viewat[Z] = viewpos[Z]+viewvec[Z];

   if(rbut_state == 0)
   {
	   /* rotate around y-axis */
	   if(delta_x != 0)
	   {
		   //axis[X]=0; axis[Y]=1; axis[Z]=0; /* rotate around y-axis (normalized)*/
		   /* calculate screen-y-axis */
		   V3CROSS(tmpvec, viewvec, viewup);
		   V3CROSS(axis, tmpvec, viewvec);
		   V3NORMALIZEF(axis);
		   M33RotationAxis(axis,delta_x*0.1*speed,rotmat); /* create the rotation matrix */
		   V3SET(tmpvec,viewvec);
		   M33MULTVEC(viewvec, rotmat, tmpvec);/* update viewvec */
	   }
	   V3CROSS(axis, viewvec, viewup); /* calculate x-axis */
	   V3CROSS(viewup, axis, viewvec); /* update viewup (only needed if delta_x != 0) */

	   if(delta_y != 0)
	   {
		   V3SET(tmpvec,viewvec);
		   V3NORMALIZEF(axis); /* M3RotationAxis requires normalized axis */
		   M33RotationAxis(axis,delta_y*0.1f*speed,rotmat); /* create the rotation matrix */
		   M33MULTVEC(viewvec, rotmat, tmpvec);/* update viewvec */
		   /* we want to update viewup */
		   V3CROSS(axis, viewvec, viewup); /* calculate x-axis */
		   V3CROSS(viewup, axis, viewvec); /* update viewup */
	   }

	   /* remove rolling 
	   axis[0]=0; axis[1]=1; axis[2]=0; /* y-axis 
	   if((len = V3DOT(viewup, axis)) > 0.0000001) /* if viewup and y-axis almost orthogonal 
	   {
		   /*project viewup on y-axis, normalize and set as new viewup 
		   V3SET(viewup, axis);
	   }
	   else if(len < 0.0000001)
	   {
		   viewup[X]=0; viewup[Y]=-1; viewup[Z]=0;
	   }
	   else*/
		   V3NORMALIZEF(viewup); /* we need to normalize, otherwise it escalates in length */ 
	   
	   viewpos[X] = viewat[X] - viewvec[X];
	   viewpos[Y] = viewat[Y] - viewvec[Y];
	   viewpos[Z] = viewat[Z] - viewvec[Z];
   } 

   gluLookAt(viewpos[X],viewpos[Y],viewpos[Z],
	   viewat[X],viewat[Y],viewat[Z],
	   viewup[X],viewup[Y],viewup[Z]);
   
   glutPostRedisplay();

   prev_x = x;
   prev_y = y;

}

int main(int argc, char *argv[])
{
   /* read texture */
   floorTexture=readPPM("balls4.ppm");
   if(!floorTexture)
   {
      printf("Could not read texture. Exiting\n");
      exit(1);
   }

   /* init GLUT */
   glutInit(&argc, argv);
   /* open window of size 800x600 with double buffering, RGB colors, and Z-buffering */
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
   glutInitWindowSize(800,600);
   glutCreateWindow("RTR: Hemuppgift 2");
   
   /* make sure we use the entire window */
   glViewport(0, 0, 800, 600);      

   /* set projection matrix */
   glMatrixMode(GL_PROJECTION);  
   glLoadIdentity();           /* reset */
   gluPerspective(45.0,800.0/600.0, 0.2,30.0); /* FOV, aspect ratio, near, far */

   glShadeModel(GL_SMOOTH);
   glEnable(GL_DEPTH_TEST);  /* enable Z-buffering */

   /* set the view matrix */
   glMatrixMode(GL_MODELVIEW);  
   glLoadIdentity();           /* reset */

   gluLookAt(0,0.5,8,  /* from */
	     0,0.0,0,    /* at */
	     0,1,0);   /* up */

   glDisable(GL_CULL_FACE);   /* show back faces of triangle */

   setupTexture(floorTexture);

   /* the diplay function is called once when the gluMainLoop is called,
    * but also each time the window has to be redrawn due to window 
    * changes (overlap, resize, etc). It can also be forced to be called
    * by making a glutPostRedisplay() call 
    */
   glutKeyboardFunc(handleKeys);
   glutSpecialFunc(handleSpecialKeys);
   glutDisplayFunc(display);
   glutMouseFunc(mouse);
   glutMotionFunc(motion);

   glutMainLoop();  /* start the program main loop */
      
   return 0;          
}
